Package: qcaERT
Title: Enhanced Robustness Tests for Qualitative Comparative Analysis
Version: 0.1.0
Authors@R: 
    person("Breno A. H.", "Marisguia", email = "marisguiabreno@gmail.com",
           role = c("aut", "cre", "cph"))
Description: Provides functions for assessing and visualizing robustness in
    Qualitative Comparative Analysis (QCA) workflows built with the 'QCA'
    package, including calibration thresholds, inclusion cutoffs, frequency
    cutoffs, case influence, subsample stability, alternative analysis settings,
    theory-specific condition sets, cluster-specific patterns, and solution
    summaries. Methods build on Dusa (2019)
    <doi:10.1007/978-3-319-75668-4> and Ragin (2014, ISBN:9780520280038).
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr, rmarkdown
Imports: methods, QCA, stats, utils
Suggests: ggplot2, knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-19 15:11:56 UTC; breno
Author: Breno A. H. Marisguia [aut, cre, cph]
Maintainer: Breno A. H. Marisguia <marisguiabreno@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-27 10:00:09 UTC
Built: R 4.5.2; ; 2026-05-27 11:13:05 UTC; unix
