# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,altset_test)
S3method(as.data.frame,calib_test)
S3method(as.data.frame,cluster_test)
S3method(as.data.frame,incl_test)
S3method(as.data.frame,loo_test)
S3method(as.data.frame,ncut_test)
S3method(as.data.frame,subsample_test)
S3method(as.data.frame,theory_test)
S3method(plot,calib_test)
S3method(plot,incl_test)
S3method(plot,theory_test)
S3method(print,altset_test)
S3method(print,calib_test)
S3method(print,cluster_test)
S3method(print,incl_test)
S3method(print,loo_test)
S3method(print,ncut_test)
S3method(print,subsample_test)
S3method(print,theory_test)
export(altset.test)
export(calib.test)
export(cluster.test)
export(incl.test)
export(loo.test)
export(ncut.test)
export(sol.chart)
export(sol.df)
export(subsample.test)
export(theory.test)
importFrom(stats,setNames)
importFrom(stats,xtabs)
