## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## ----core-rule----------------------------------------------------------------
# print(x)
# as.data.frame(x)
# x$results
# x$diagnostics

## ----boundary-layout----------------------------------------------------------
# as.data.frame(incl_out)
# incl_out$diagnostics
# incl_out$bounds
# 
# as.data.frame(ncut_out)
# ncut_out$diagnostics
# ncut_out$bounds
# 
# as.data.frame(calib_out)
# calib_out$diagnostics
# calib_out$bounds

## ----case-sample-layout-------------------------------------------------------
# as.data.frame(loo_out)
# loo_out$diagnostics
# loo_out$by_case
# 
# as.data.frame(subsample_out)
# subsample_out$diagnostics
# subsample_out$by_run
# subsample_out$summary

## ----altset-layout------------------------------------------------------------
# as.data.frame(altset_out)
# altset_out$diagnostics
# altset_out$by_draw
# altset_out$summary

## ----cluster-layout-----------------------------------------------------------
# cluster_out$results$overview
# cluster_out$results$clusters
# cluster_out$results$units
# 
# as.data.frame(cluster_out)
# cluster_out$diagnostics

## ----theory-layout------------------------------------------------------------
# as.data.frame(theory_out)
# theory_out$results$models
# theory_out$results$solutions
# theory_out$results$pairwise
# theory_out$diagnostics
# theory_out$by_theory

## ----sol-df-layout------------------------------------------------------------
# tab <- sol.df(intermediate = sol, solution = "intermediate")
# tab
# is.data.frame(tab)

